package cc.fascinated.bat.features.logging.listeners;

import cc.fascinated.bat.common.EmbedDescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.logging.LogFeature;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.DiscordService;
import cc.fascinated.bat.service.GuildService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.events.guild.GuildBanEvent;
import net.dv8tion.jda.api.events.guild.GuildUnbanEvent;
import net.dv8tion.jda.api.events.guild.member.GuildMemberJoinEvent;
import net.dv8tion.jda.api.events.guild.member.GuildMemberRemoveEvent;
import net.dv8tion.jda.api.events.guild.member.GuildMemberRoleAddEvent;
import net.dv8tion.jda.api.events.guild.member.GuildMemberRoleRemoveEvent;
import net.dv8tion.jda.api.events.guild.member.update.GuildMemberUpdateNicknameEvent;
import net.dv8tion.jda.api.events.guild.member.update.GuildMemberUpdateTimeOutEvent;
import net.dv8tion.jda.api.events.user.update.UserUpdateAvatarEvent;
import net.dv8tion.jda.api.events.user.update.UserUpdateGlobalNameEvent;
import net.dv8tion.jda.api.events.user.update.UserUpdateNameEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

import java.time.OffsetDateTime;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Component
public class MemberListener implements EventListener {
    private final LogFeature logFeature;
    private final GuildService guildService;

    @Autowired
    public MemberListener(@NonNull ApplicationContext context, @NonNull GuildService guildService) {
        this.logFeature = context.getBean(LogFeature.class);
        this.guildService = guildService;
    }

    @Override
    public void onGuildMemberJoin(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull GuildMemberJoinEvent event) {
        if (user.getDiscordUser().isBot()) return;

        logFeature.sendLog(guild, LogType.MEMBER_JOIN, EmbedUtils.successEmbed()
                .setDescription(new EmbedDescriptionBuilder("Member Joined")
                        .appendLine("Member: %s".formatted(user.getDiscordUser().getAsMention()), true)
                        .appendLine("Username: %s".formatted(user.getDiscordUser().getName()), true)
                        .appendLine("Account Age: <t:%s:R>".formatted(user.getDiscordUser().getTimeCreated().toEpochSecond()), true)
                        .build())
                        .setThumbnail(user.getDiscordUser().getEffectiveAvatarUrl())
                .build());
    }

    @Override
    public void onGuildMemberLeave(@NonNull BatGuild guild, BatUser user, @NonNull GuildMemberRemoveEvent event) {
        if (user.getDiscordUser().isBot()) return;

        logFeature.sendLog(guild, LogType.MEMBER_LEAVE, EmbedUtils.errorEmbed()
                .setDescription(new EmbedDescriptionBuilder("Member Left")
                        .appendLine("Member: %s".formatted(user.getDiscordUser().getAsMention()), true)
                        .appendLine("Username: %s".formatted(user.getDiscordUser().getName()), true)
                        .build())
                .build());
    }

    @Override
    public void onGuildMemberUpdateNickname(@NonNull BatGuild guild, @NonNull BatUser user, String oldName, String newName, @NonNull GuildMemberUpdateNicknameEvent event) {
        if (user.getDiscordUser().isBot()) return;

        logFeature.sendLog(guild, LogType.MEMBER_NICKNAME_UPDATE, EmbedUtils.genericEmbed()
                .setDescription(new EmbedDescriptionBuilder("Member Nickname Updated")
                        .appendLine("Member: %s".formatted(user.getDiscordUser().getAsMention()), true)
                        .appendLine("Old Nickname: `%s`".formatted(oldName == null ? user.getName() : oldName), true)
                        .appendLine("New Nickname: `%s`".formatted(newName == null ? "Removed Nickname" : newName), true)
                        .build())
                .build());
    }

    @Override
    public void onUserUpdateGlobalName(@NonNull BatUser user, String oldName, String newName, @NonNull UserUpdateGlobalNameEvent event) {
        if (user.getDiscordUser().isBot()) return;
        for (Guild guild : DiscordService.JDA.getGuilds()) {
            BatGuild batGuild = guildService.getGuild(guild.getId());
            if (batGuild == null) continue;

            logFeature.sendLog(batGuild, LogType.MEMBER_GLOBAL_NAME_UPDATE, EmbedUtils.genericEmbed()
                    .setDescription(new EmbedDescriptionBuilder("Member Name Updated")
                            .appendLine("Member: %s".formatted(user.getDiscordUser().getAsMention()), true)
                            .appendLine("Old Name: `%s`".formatted(oldName == null ? user.getName() : oldName), true)
                            .appendLine("New Name: `%s`".formatted(newName == null ? "Removed Name" : newName), true)
                            .build())
                    .build());
        }
    }

    @Override
    public void onUserUpdateName(@NonNull BatUser user, String oldName, String newName, @NonNull UserUpdateNameEvent event) {
        if (user.getDiscordUser().isBot()) return;
        for (Guild guild : DiscordService.JDA.getGuilds()) {
            BatGuild batGuild = guildService.getGuild(guild.getId());
            if (batGuild == null) continue;

            logFeature.sendLog(batGuild, LogType.MEMBER_USERNAME_UPDATE, EmbedUtils.genericEmbed()
                    .setDescription(new EmbedDescriptionBuilder("Member Username Updated")
                            .appendLine("Member: %s".formatted(user.getDiscordUser().getAsMention()), true)
                            .appendLine("Old Username: `%s`".formatted(oldName), true)
                            .appendLine("New Username: `%s`".formatted(newName), true)
                            .build())
                    .build());
        }
    }

    @Override
    public void onUserUpdateAvatar(@NonNull BatUser user, String oldAvatarUrl, String newAvatarUrl, @NonNull UserUpdateAvatarEvent event) {
        if (user.getDiscordUser().isBot()) return;
        for (Guild guild : DiscordService.JDA.getGuilds()) {
            BatGuild batGuild = guildService.getGuild(guild.getId());
            if (batGuild == null) continue;

            logFeature.sendLog(batGuild, LogType.MEMBER_USERNAME_UPDATE, EmbedUtils.genericEmbed()
                    .setDescription(new EmbedDescriptionBuilder("Member Avatar Updated")
                            .appendLine("Member: %s".formatted(user.getDiscordUser().getAsMention()), true)
                            .appendLine("Old Avatar: [avatar](%s)".formatted(oldAvatarUrl), true)
                            .appendLine("New Avatar: [avatar](%s)".formatted(newAvatarUrl), true)
                            .build())
                    .build());
        }
    }

    @Override
    public void onGuildMemberRoleAdd(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull List<Role> rolesAdded, @NonNull GuildMemberRoleAddEvent event) {
        if (user.getDiscordUser().isBot()) return;

        StringBuilder roles = new StringBuilder();
        for (Role role : rolesAdded) {
            roles.append(role.getAsMention()).append(", ");
        }

        String s = rolesAdded.size() > 1 ? "s" : "";
        logFeature.sendLog(guild, LogType.MEMBER_ROLE_UPDATE, EmbedUtils.successEmbed()
                .setDescription(new EmbedDescriptionBuilder("Member Role%s Added".formatted(s))
                        .appendLine("Member: %s".formatted(user.getDiscordUser().getAsMention()), true)
                        .appendLine("Role%s Added: %s".formatted(s, roles.substring(0, roles.length() - 2)), true)
                        .build())
                .build());
    }

    @Override
    public void onGuildMemberRoleRemove(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull List<Role> rolesAdded, @NonNull GuildMemberRoleRemoveEvent event) {
        if (user.getDiscordUser().isBot()) return;

        StringBuilder roles = new StringBuilder();
        for (Role role : rolesAdded) {
            roles.append(role.getAsMention()).append(", ");
        }

        String s = rolesAdded.size() > 1 ? "s" : "";
        logFeature.sendLog(guild, LogType.MEMBER_ROLE_UPDATE, EmbedUtils.errorEmbed()
                .setDescription(new EmbedDescriptionBuilder("Member Role%s Removed".formatted(s))
                        .appendLine("Member: %s".formatted(user.getDiscordUser().getAsMention()), true)
                        .appendLine("Role%s Removed: %s".formatted(s, roles.substring(0, roles.length() - 2)), true)
                        .build())
                .build());
    }

    @Override
    public void onGuildMemberBan(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull GuildBanEvent event) {
        if (user.getDiscordUser().isBot()) return;

        logFeature.sendLog(guild, LogType.MEMBER_BAN, EmbedUtils.errorEmbed()
                .setDescription(new EmbedDescriptionBuilder("Member Banned")
                        .appendLine("Member: %s".formatted(user.getDiscordUser().getAsMention()), true)
                        .build())
                .build());
    }

    @Override
    public void onGuildMemberUnban(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull GuildUnbanEvent event) {
        if (user.getDiscordUser().isBot()) return;

        logFeature.sendLog(guild, LogType.MEMBER_UNBAN, EmbedUtils.successEmbed()
                .setDescription(new EmbedDescriptionBuilder("Member Unbanned")
                        .appendLine("Member: %s".formatted(user.getDiscordUser().getAsMention()), true)
                        .build())
                .build());
    }

    @Override
    public void onGuildMemberTimeout(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull GuildMemberUpdateTimeOutEvent event) {
        OffsetDateTime timeoutEnd = event.getNewTimeOutEnd();
        if (user.getDiscordUser().isBot() || timeoutEnd == null) return;

        long seconds = timeoutEnd.toInstant().getEpochSecond();
        logFeature.sendLog(guild, LogType.MEMBER_TIMEOUT, EmbedUtils.errorEmbed()
                .setDescription(new EmbedDescriptionBuilder("Member Timed Out")
                        .appendLine("Member: %s".formatted(user.getDiscordUser().getAsMention()), true)
                        .appendLine("Timeout End: <t:%s>".formatted(seconds), true)
                        .appendLine("Relative End: <t:%s:R>".formatted(seconds), true)
                        .build())
                .build());
    }
}
